@@ -493,21 +493,21 @@ async def send(self, msg, toUserName=None, mediaId=None):
             'Ret': -1005, }})
     elif msg[:5] == '@fil@':
         if mediaId is None:
-            r = self.send_file(msg[5:], toUserName)
+            r = await self.send_file(msg[5:], toUserName)
         else:
-            r = self.send_file(msg[5:], toUserName, mediaId)
+            r = await self.send_file(msg[5:], toUserName, mediaId)
     elif msg[:5] == '@img@':
         if mediaId is None:
-            r = self.send_image(msg[5:], toUserName)
+            r = await self.send_image(msg[5:], toUserName)
         else:
-            r = self.send_image(msg[5:], toUserName, mediaId)
+            r = await self.send_image(msg[5:], toUserName, mediaId)
     elif msg[:5] == '@msg@':
-        r = self.send_msg(msg[5:], toUserName)
+        r = await self.send_msg(msg[5:], toUserName)
     elif msg[:5] == '@vid@':
         if mediaId is None:
-            r = self.send_video(msg[5:], toUserName)
+            r = await self.send_video(msg[5:], toUserName)
         else:
-            r = self.send_video(msg[5:], toUserName, mediaId)
+            r = await self.send_video(msg[5:], toUserName, mediaId)
     else:
         r = await self.send_msg(msg, toUserName)
     return r
